@@ -43,8 +43,6 @@ using namespace std;
 
 #include "include/assert.h"
 
-#include "erasure-code/ErasureCodePlugin.h"
-
 #define dout_subsys ceph_subsys_mon
 
 Monitor *mon = NULL;
@@ -186,21 +184,6 @@ void usage()
   generic_server_usage();
 }
 
-int preload_erasure_code()
-{
-  string directory = g_conf->osd_pool_default_erasure_code_directory;
-  string plugins = g_conf->osd_erasure_code_plugins;
-  stringstream ss;
-  int r = ErasureCodePluginRegistry::instance().preload(plugins,
-							directory,
-							ss);
-  if (r)
-    derr << ss.str() << dendl;
-  else
-    dout(10) << ss.str() << dendl;
-  return r;
-}
-
 int main(int argc, const char **argv) 
 {
   int err;
@@ -423,7 +406,8 @@ int main(int argc, const char **argv)
   // screwing us over
   Preforker prefork;
   if (!(flags & CINIT_FLAG_NO_DAEMON_ACTIONS)) {
-    if (global_init_prefork(g_ceph_context, 0) >= 0) {
+    if (g_conf->daemonize) {
+      global_init_prefork(g_ceph_context, 0);
       prefork.prefork();
       if (prefork.is_parent()) {
 	return prefork.parent_wait();
@@ -432,8 +416,6 @@ int main(int argc, const char **argv)
     }
     common_init_finish(g_ceph_context);
     global_init_chdir(g_ceph_context);
-    if (preload_erasure_code() < -1)
-      prefork.exit(1);
   }
 
   MonitorDBStore *store = new MonitorDBStore(g_conf->mon_data);
@@ -470,7 +452,7 @@ int main(int argc, const char **argv)
   bufferlist magicbl;
   err = store->get(Monitor::MONITOR_NAME, "magic", magicbl);
   if (!magicbl.length()) {
-    derr << "unable to read magic from mon data.. did you run mkcephfs?" << dendl;
+    derr << "unable to read magic from mon data" << dendl;
     prefork.exit(1);
   }
   string magic(magicbl.c_str(), magicbl.length()-1);  // ignore trailing \n
