@@ -16,7 +16,6 @@
 #define CEPH_MON_TYPES_H
 
 #include "include/utime.h"
-#include "include/util.h"
 #include "common/Formatter.h"
 
 #define PAXOS_PGMAP      0  // before osd, for pg kick to behave
@@ -85,55 +84,49 @@ struct LevelDBStoreStats {
     DECODE_FINISH(p);
   }
 };
-WRITE_CLASS_ENCODER(LevelDBStoreStats);
+WRITE_CLASS_ENCODER(LevelDBStoreStats)
 
 // data stats
 
 struct DataStats {
-  ceph_data_stats_t fs_stats;
   // data dir
+  uint64_t kb_total;
+  uint64_t kb_used;
+  uint64_t kb_avail;
+  int latest_avail_percent;
   utime_t last_update;
+
   LevelDBStoreStats store_stats;
 
   void dump(Formatter *f) const {
     assert(f != NULL);
-    f->dump_int("kb_total", (fs_stats.byte_total/1024));
-    f->dump_int("kb_used", (fs_stats.byte_used/1024));
-    f->dump_int("kb_avail", (fs_stats.byte_avail/1024));
-    f->dump_int("avail_percent", fs_stats.avail_percent);
+    f->dump_int("kb_total", kb_total);
+    f->dump_int("kb_used", kb_used);
+    f->dump_int("kb_avail", kb_avail);
+    f->dump_int("avail_percent", latest_avail_percent);
     f->dump_stream("last_updated") << last_update;
+
     f->open_object_section("store_stats");
     store_stats.dump(f);
     f->close_section();
   }
 
   void encode(bufferlist &bl) const {
-    ENCODE_START(3, 1, bl);
-    ::encode(fs_stats.byte_total, bl);
-    ::encode(fs_stats.byte_used, bl);
-    ::encode(fs_stats.byte_avail, bl);
-    ::encode(fs_stats.avail_percent, bl);
+    ENCODE_START(2, 1, bl);
+    ::encode(kb_total, bl);
+    ::encode(kb_used, bl);
+    ::encode(kb_avail, bl);
+    ::encode(latest_avail_percent, bl);
     ::encode(last_update, bl);
     ::encode(store_stats, bl);
     ENCODE_FINISH(bl);
   }
   void decode(bufferlist::iterator &p) {
     DECODE_START(1, p);
-    // we moved from having fields in kb to fields in byte
-    if (struct_v > 2) {
-      ::decode(fs_stats.byte_total, p);
-      ::decode(fs_stats.byte_used, p);
-      ::decode(fs_stats.byte_avail, p);
-    } else {
-      uint64_t t;
-      ::decode(t, p);
-      fs_stats.byte_total = t*1024;
-      ::decode(t, p);
-      fs_stats.byte_used = t*1024;
-      ::decode(t, p);
-      fs_stats.byte_avail = t*1024;
-    }
-    ::decode(fs_stats.avail_percent, p);
+    ::decode(kb_total, p);
+    ::decode(kb_used, p);
+    ::decode(kb_avail, p);
+    ::decode(latest_avail_percent, p);
     ::decode(last_update, p);
     if (struct_v > 1)
       ::decode(store_stats, p);
@@ -141,7 +134,7 @@ struct DataStats {
     DECODE_FINISH(p);
   }
 };
-WRITE_CLASS_ENCODER(DataStats);
+WRITE_CLASS_ENCODER(DataStats)
 
 struct ScrubResult {
   map<string,uint32_t> prefix_crc;  ///< prefix -> crc
@@ -180,7 +173,7 @@ struct ScrubResult {
     ls.back()->prefix_keys["bar"] = 456;
   }
 };
-WRITE_CLASS_ENCODER(ScrubResult);
+WRITE_CLASS_ENCODER(ScrubResult)
 
 static inline ostream& operator<<(ostream& out, const ScrubResult& r) {
   return out << "ScrubResult(keys " << r.prefix_keys << " crc " << r.prefix_crc << ")";
