@@ -781,7 +781,7 @@ void OSDMap::Incremental::dump(Formatter *f) const
   f->open_array_section("old_erasure_code_profiles");
   for (vector<string>::const_iterator p = old_erasure_code_profiles.begin();
        p != old_erasure_code_profiles.end();
-       p++) {
+       ++p) {
     f->dump_string("old", p->c_str());
   }
   f->close_section();
@@ -950,7 +950,7 @@ bool OSDMap::find_osd_on_ip(const entity_addr_t& ip) const
 }
 
 
-uint64_t OSDMap::get_features(int entity_type, uint64_t *pmask) const
+uint64_t OSDMap::get_features(uint64_t *pmask) const
 {
   uint64_t features = 0;  // things we actually have
   uint64_t mask = 0;      // things we could have
@@ -959,7 +959,10 @@ uint64_t OSDMap::get_features(int entity_type, uint64_t *pmask) const
     features |= CEPH_FEATURE_CRUSH_TUNABLES;
   if (crush->has_nondefault_tunables2())
     features |= CEPH_FEATURE_CRUSH_TUNABLES2;
-  if (crush->has_nondefault_tunables3())
+  if (crush->has_v2_rules())
+    features |= CEPH_FEATURE_CRUSH_V2;
+  if (crush->has_nondefault_tunables3() ||
+      crush->has_v3_rules())
     features |= CEPH_FEATURE_CRUSH_TUNABLES3;
   mask |= CEPH_FEATURES_CRUSH;
 
@@ -967,27 +970,16 @@ uint64_t OSDMap::get_features(int entity_type, uint64_t *pmask) const
     if (p->second.flags & pg_pool_t::FLAG_HASHPSPOOL) {
       features |= CEPH_FEATURE_OSDHASHPSPOOL;
     }
-    if (p->second.is_erasure() &&
-	entity_type != CEPH_ENTITY_TYPE_CLIENT) { // not for clients
+    if (p->second.is_erasure()) {
       features |= CEPH_FEATURE_OSD_ERASURE_CODES;
     }
     if (!p->second.tiers.empty() ||
 	p->second.is_tier()) {
       features |= CEPH_FEATURE_OSD_CACHEPOOL;
     }
-    int ruleid = crush->find_rule(p->second.get_crush_ruleset(),
-				  p->second.get_type(),
-				  p->second.get_size());
-    if (ruleid >= 0) {
-      if (crush->is_v2_rule(ruleid))
-	features |= CEPH_FEATURE_CRUSH_V2;
-      if (crush->is_v3_rule(ruleid))
-	features |= CEPH_FEATURE_CRUSH_TUNABLES3;
-    }
   }
-  mask |= CEPH_FEATURE_OSDHASHPSPOOL | CEPH_FEATURE_OSD_CACHEPOOL;
-  if (entity_type != CEPH_ENTITY_TYPE_CLIENT)
-    mask |= CEPH_FEATURE_OSD_ERASURE_CODES;
+  mask |= CEPH_FEATURE_OSDHASHPSPOOL | CEPH_FEATURE_OSD_CACHEPOOL |
+          CEPH_FEATURE_OSD_ERASURE_CODES;
 
   if (osd_primary_affinity) {
     for (int i = 0; i < max_osd; ++i) {
@@ -1222,18 +1214,18 @@ int OSDMap::apply_incremental(const Incremental &inc)
   }
 
   // erasure_code_profiles
+  for (map<string,map<string,string> >::const_iterator i =
+	 inc.new_erasure_code_profiles.begin();
+       i != inc.new_erasure_code_profiles.end();
+       ++i) {
+    set_erasure_code_profile(i->first, i->second);
+  }
+  
   for (vector<string>::const_iterator i = inc.old_erasure_code_profiles.begin();
        i != inc.old_erasure_code_profiles.end();
        ++i)
     erasure_code_profiles.erase(*i);
   
-  for (map<string,map<string,string> >::const_iterator i =
-	 inc.new_erasure_code_profiles.begin();
-       i != inc.new_erasure_code_profiles.end();
-       i++) {
-    set_erasure_code_profile(i->first, i->second);
-  }
-  
   // up/down
   for (map<int32_t,uint8_t>::const_iterator i = inc.new_state.begin();
        i != inc.new_state.end();
@@ -1807,15 +1799,7 @@ void OSDMap::encode(bufferlist& bl, uint64_t features) const
     ENCODE_START(1, 1, bl); // extended, osd-only data
     ::encode(osd_addrs->hb_back_addr, bl);
     ::encode(osd_info, bl);
-    {
-      // put this in a sorted, ordered map<> so that we encode in a
-      // deterministic order.
-      map<entity_addr_t,utime_t> blacklist_map;
-      for (ceph::unordered_map<entity_addr_t,utime_t>::const_iterator p =
-	     blacklist.begin(); p != blacklist.end(); ++p)
-	blacklist_map.insert(make_pair(p->first, p->second));
-      ::encode(blacklist_map, bl);
-    }
+    ::encode(blacklist, bl);
     ::encode(osd_addrs->cluster_addr, bl);
     ::encode(cluster_snapshot_epoch, bl);
     ::encode(cluster_snapshot, bl);
@@ -2044,11 +2028,11 @@ void OSDMap::dump_erasure_code_profiles(const map<string,map<string,string> > &p
   f->open_object_section("erasure_code_profiles");
   for (map<string,map<string,string> >::const_iterator i = profiles.begin();
        i != profiles.end();
-       i++) {
+       ++i) {
     f->open_object_section(i->first.c_str());
     for (map<string,string>::const_iterator j = i->second.begin();
 	 j != i->second.end();
-	 j++) {
+	 ++j) {
       f->dump_string(j->first.c_str(), j->second.c_str());
     }
     f->close_section();
@@ -2173,7 +2157,6 @@ void OSDMap::generate_test_instances(list<OSDMap*>& o)
   uuid_d fsid;
   o.back()->build_simple(cct, 1, fsid, 16, 7, 8);
   o.back()->created = o.back()->modified = utime_t(1, 2);  // fix timestamp
-  o.back()->blacklist[entity_addr_t()] = utime_t(5, 6);
   cct->put();
 }
 
@@ -2526,18 +2509,8 @@ int OSDMap::build_simple(CephContext *cct, epoch_t e, uuid_d &fsid,
   pool_names.push_back("metadata");
   pool_names.push_back("rbd");
 
-  stringstream ss;
-  int r;
-  if (nosd >= 0)
-    r = build_simple_crush_map(cct, *crush, nosd, &ss);
-  else
-    r = build_simple_crush_map_from_conf(cct, *crush, &ss);
-
   int poolbase = get_max_osd() ? get_max_osd() : 1;
 
-  int const default_replicated_ruleset = crush->get_osd_pool_default_crush_replicated_ruleset(cct);
-  assert(default_replicated_ruleset >= 0);
-
   for (vector<string>::iterator p = pool_names.begin();
        p != pool_names.end(); ++p) {
     int64_t pool = ++pool_max;
@@ -2547,7 +2520,8 @@ int OSDMap::build_simple(CephContext *cct, epoch_t e, uuid_d &fsid,
       pools[pool].flags |= pg_pool_t::FLAG_HASHPSPOOL;
     pools[pool].size = cct->_conf->osd_pool_default_size;
     pools[pool].min_size = cct->_conf->get_osd_pool_default_min_size();
-    pools[pool].crush_ruleset = default_replicated_ruleset;
+    pools[pool].crush_ruleset =
+      CrushWrapper::get_osd_pool_default_crush_replicated_ruleset(cct);
     pools[pool].object_hash = CEPH_STR_HASH_RJENKINS;
     pools[pool].set_pg_num(poolbase << pg_bits);
     pools[pool].set_pgp_num(poolbase << pgp_bits);
@@ -2558,6 +2532,13 @@ int OSDMap::build_simple(CephContext *cct, epoch_t e, uuid_d &fsid,
     name_pool[*p] = pool;
   }
 
+  stringstream ss;
+  int r;
+  if (nosd >= 0)
+    r = build_simple_crush_map(cct, *crush, nosd, &ss);
+  else
+    r = build_simple_crush_map_from_conf(cct, *crush, &ss);
+
   if (r < 0)
     lderr(cct) << ss.str() << dendl;
   
@@ -2566,25 +2547,13 @@ int OSDMap::build_simple(CephContext *cct, epoch_t e, uuid_d &fsid,
     set_weight(i, CEPH_OSD_OUT);
   }
 
-  map<string,string> profile_map;
-  r = get_erasure_code_profile_default(cct, profile_map, &ss);
-  if (r < 0) {
-    lderr(cct) << ss.str() << dendl;
-    return r;
-  }
-  set_erasure_code_profile("default", profile_map);
-  return 0;
-}
-
-int OSDMap::get_erasure_code_profile_default(CephContext *cct,
-					     map<string,string> &profile_map,
-					     ostream *ss)
-{
-  int r = get_str_map(cct->_conf->osd_pool_default_erasure_code_profile,
-		      *ss,
-		      &profile_map);
-  profile_map["directory"] =
+  map<string,string> erasure_code_profile_map;
+  r = get_str_map(cct->_conf->osd_pool_default_erasure_code_profile,
+		  ss,
+		  &erasure_code_profile_map);
+  erasure_code_profile_map["directory"] =
     cct->_conf->osd_pool_default_erasure_code_directory;
+  set_erasure_code_profile("default", erasure_code_profile_map);
   return r;
 }
 
