@@ -447,25 +447,6 @@ public:
   eversion_t  last_complete_ondisk;  // last_complete that has committed.
   eversion_t  last_update_applied;
 
-
-  struct C_UpdateLastRollbackInfoTrimmedToApplied : Context {
-    PGRef pg;
-    epoch_t e;
-    eversion_t v;
-    C_UpdateLastRollbackInfoTrimmedToApplied(PG *pg, epoch_t e, eversion_t v)
-      : pg(pg), e(e), v(v) {}
-    void finish(int) {
-      pg->lock();
-      if (!pg->pg_has_reset_since(e)) {
-	pg->last_rollback_info_trimmed_to_applied = v;
-      }
-      pg->unlock();
-    }
-  };
-  // entries <= last_rollback_info_trimmed_to_applied have been trimmed,
-  // and the transaction has applied
-  eversion_t  last_rollback_info_trimmed_to_applied;
-
   // primary state
  public:
   pg_shard_t primary;
@@ -506,12 +487,6 @@ public:
 
 
 public:    
-  struct BufferedRecoveryMessages {
-    map<int, map<spg_t, pg_query_t> > query_map;
-    map<int, vector<pair<pg_notify_t, pg_interval_map_t> > > info_map;
-    map<int, vector<pair<pg_notify_t, pg_interval_map_t> > > notify_list;
-  };
-
   struct RecoveryCtx {
     utime_t start_time;
     map<int, map<spg_t, pg_query_t> > *query_map;
@@ -533,48 +508,6 @@ public:
 	on_applied(on_applied),
 	on_safe(on_safe),
 	transaction(transaction) {}
-
-    RecoveryCtx(BufferedRecoveryMessages &buf, RecoveryCtx &rctx)
-      : query_map(&(buf.query_map)),
-	info_map(&(buf.info_map)),
-	notify_list(&(buf.notify_list)),
-	on_applied(rctx.on_applied),
-	on_safe(rctx.on_safe),
-	transaction(rctx.transaction) {}
-
-    void accept_buffered_messages(BufferedRecoveryMessages &m) {
-      assert(query_map);
-      assert(info_map);
-      assert(notify_list);
-      for (map<int, map<spg_t, pg_query_t> >::iterator i = m.query_map.begin();
-	   i != m.query_map.end();
-	   ++i) {
-	map<spg_t, pg_query_t> &omap = (*query_map)[i->first];
-	for (map<spg_t, pg_query_t>::iterator j = i->second.begin();
-	     j != i->second.end();
-	     ++j) {
-	  omap[j->first] = j->second;
-	}
-      }
-      for (map<int, vector<pair<pg_notify_t, pg_interval_map_t> > >::iterator i
-	     = m.info_map.begin();
-	   i != m.info_map.end();
-	   ++i) {
-	vector<pair<pg_notify_t, pg_interval_map_t> > &ovec =
-	  (*info_map)[i->first];
-	ovec.reserve(ovec.size() + i->second.size());
-	ovec.insert(ovec.end(), i->second.begin(), i->second.end());
-      }
-      for (map<int, vector<pair<pg_notify_t, pg_interval_map_t> > >::iterator i
-	     = m.notify_list.begin();
-	   i != m.notify_list.end();
-	   ++i) {
-	vector<pair<pg_notify_t, pg_interval_map_t> > &ovec =
-	  (*notify_list)[i->first];
-	ovec.reserve(ovec.size() + i->second.size());
-	ovec.insert(ovec.end(), i->second.begin(), i->second.end());
-      }
-    }
   };
 
   struct NamedState {
@@ -583,7 +516,7 @@ public:
     const char *get_state_name() { return state_name; }
     NamedState(CephContext *cct_, const char *state_name_)
       : state_name(state_name_),
-        enter_time(ceph_clock_now(cct_)) {};
+        enter_time(ceph_clock_now(cct_)) {}
     virtual ~NamedState() {}
   };
 
@@ -1014,8 +947,7 @@ public:
       waiting_on(0), shallow_errors(0), deep_errors(0), fixed(0),
       active_rep_scrub(0),
       must_scrub(false), must_deep_scrub(false), must_repair(false),
-      classic(false),
-      finalizing(false), is_chunky(false), state(INACTIVE),
+      state(INACTIVE),
       deep(false)
     {
     }
@@ -1050,12 +982,7 @@ public:
     // Map from object with errors to good peer
     map<hobject_t, pair<ScrubMap::object, pg_shard_t> > authoritative;
 
-    // classic scrub
-    bool classic;
-    bool finalizing;
-
     // chunky scrub
-    bool is_chunky;
     hobject_t start, end;
     eversion_t subset_last_update;
 
@@ -1112,9 +1039,6 @@ public:
       if (!block_writes)
 	return false;
 
-      if (!is_chunky)
-	return true;
-
       if (soid >= start && soid < end)
 	return true;
 
@@ -1123,8 +1047,6 @@ public:
 
     // clear all state
     void reset() {
-      classic = false;
-      finalizing = false;
       block_writes = false;
       active = false;
       queue_snap_trim = false;
@@ -1175,9 +1097,6 @@ public:
   void scrub_clear_state();
   bool scrub_gather_replica_maps();
   void _scan_snaps(ScrubMap &map);
-  void _scan_rollback_obs(
-    const vector<ghobject_t> &rollback_obs,
-    ThreadPool::TPHandle &handle);
   void _request_scrub_map_classic(pg_shard_t replica, eversion_t version);
   void _request_scrub_map(pg_shard_t replica, eversion_t version,
                           hobject_t start, hobject_t end, bool deep);
@@ -1208,7 +1127,7 @@ public:
     const hobject_t &hoid,
     const map<string, bufferptr> &attrs,
     pg_shard_t osd,
-    ostream &out) { return false; };
+    ostream &out) { return false; }
   void clear_scrub_reserved();
   void scrub_reserve_replicas();
   void scrub_unreserve_replicas();
@@ -1308,12 +1227,10 @@ public:
   struct MNotifyRec : boost::statechart::event< MNotifyRec > {
     pg_shard_t from;
     pg_notify_t notify;
-    uint64_t features;
-    MNotifyRec(pg_shard_t from, pg_notify_t &notify, uint64_t f) :
-      from(from), notify(notify), features(f) {}
+    MNotifyRec(pg_shard_t from, pg_notify_t &notify) :
+      from(from), notify(notify) {}
     void print(std::ostream *out) const {
-      *out << "MNotifyRec from " << from << " notify: " << notify
-        << " features: 0x" << hex << features << dec;
+      *out << "MNotifyRec from " << from << " notify: " << notify;
     }
   };
 
@@ -1405,17 +1322,10 @@ public:
 
   TrivialEvent(AllReplicasActivated)
 
-  TrivialEvent(IntervalFlush)
-
   /* Encapsulates PG recovery process */
   class RecoveryState {
     void start_handle(RecoveryCtx *new_ctx);
     void end_handle();
-  public:
-    void begin_block_outgoing();
-    void end_block_outgoing();
-    void clear_blocked_outgoing();
-  private:
 
     /* States */
     struct Initial;
@@ -1439,47 +1349,40 @@ public:
 
       /* Accessor functions for state methods */
       ObjectStore::Transaction* get_cur_transaction() {
-	assert(state->rctx);
 	assert(state->rctx->transaction);
 	return state->rctx->transaction;
       }
 
       void send_query(pg_shard_t to, const pg_query_t &query) {
-	assert(state->rctx);
 	assert(state->rctx->query_map);
 	(*state->rctx->query_map)[to.osd][spg_t(pg->info.pgid.pgid, to.shard)] =
 	  query;
       }
 
       map<int, map<spg_t, pg_query_t> > *get_query_map() {
-	assert(state->rctx);
 	assert(state->rctx->query_map);
 	return state->rctx->query_map;
       }
 
       map<int, vector<pair<pg_notify_t, pg_interval_map_t> > > *get_info_map() {
-	assert(state->rctx);
 	assert(state->rctx->info_map);
 	return state->rctx->info_map;
       }
 
       list< Context* > *get_on_safe_context_list() {
-	assert(state->rctx);
 	assert(state->rctx->on_safe);
 	return &(state->rctx->on_safe->contexts);
       }
 
       list< Context * > *get_on_applied_context_list() {
-	assert(state->rctx);
 	assert(state->rctx->on_applied);
 	return &(state->rctx->on_applied->contexts);
       }
 
-      RecoveryCtx *get_recovery_ctx() { return &*(state->rctx); }
+      RecoveryCtx *get_recovery_ctx() { return state->rctx; }
 
       void send_notify(pg_shard_t to,
 		       const pg_notify_t &info, const pg_interval_map_t &pi) {
-	assert(state->rctx);
 	assert(state->rctx->notify_list);
 	(*state->rctx->notify_list)[to.osd].push_back(make_pair(info, pi));
       }
@@ -1525,14 +1428,12 @@ public:
 	boost::statechart::custom_reaction< ActMap >,
 	boost::statechart::custom_reaction< NullEvt >,
 	boost::statechart::custom_reaction< FlushedEvt >,
-	boost::statechart::custom_reaction< IntervalFlush >,
 	boost::statechart::transition< boost::statechart::event_base, Crashed >
 	> reactions;
       boost::statechart::result react(const QueryState& q);
       boost::statechart::result react(const AdvMap&);
       boost::statechart::result react(const ActMap&);
       boost::statechart::result react(const FlushedEvt&);
-      boost::statechart::result react(const IntervalFlush&);
       boost::statechart::result react(const boost::statechart::event_base&) {
 	return discard_event();
       }
@@ -1549,13 +1450,11 @@ public:
 	boost::statechart::custom_reaction< AdvMap >,
 	boost::statechart::custom_reaction< NullEvt >,
 	boost::statechart::custom_reaction< FlushedEvt >,
-	boost::statechart::custom_reaction< IntervalFlush >,
 	boost::statechart::transition< boost::statechart::event_base, Crashed >
 	> reactions;
       boost::statechart::result react(const QueryState& q);
       boost::statechart::result react(const AdvMap&);
       boost::statechart::result react(const FlushedEvt&);
-      boost::statechart::result react(const IntervalFlush&);
       boost::statechart::result react(const boost::statechart::event_base&) {
 	return discard_event();
       }
@@ -1645,8 +1544,8 @@ public:
       Active(my_context ctx);
       void exit();
 
-      const set<pg_shard_t> remote_shards_to_reserve_recovery;
-      const set<pg_shard_t> remote_shards_to_reserve_backfill;
+      const set<pg_shard_t> sorted_actingbackfill_set;
+      const set<pg_shard_t> sorted_backfill_set;
       bool all_replicas_activated;
 
       typedef boost::mpl::list <
@@ -1725,14 +1624,10 @@ public:
 
     struct NotBackfilling : boost::statechart::state< NotBackfilling, Active>, NamedState {
       typedef boost::mpl::list<
-	boost::statechart::transition< RequestBackfill, WaitLocalBackfillReserved>,
-	boost::statechart::custom_reaction< RemoteBackfillReserved >,
-	boost::statechart::custom_reaction< RemoteReservationRejected >
+	boost::statechart::transition< RequestBackfill, WaitLocalBackfillReserved>
 	> reactions;
       NotBackfilling(my_context ctx);
       void exit();
-      boost::statechart::result react(const RemoteBackfillReserved& evt);
-      boost::statechart::result react(const RemoteReservationRejected& evt);
     };
 
     struct RepNotRecovering;
@@ -1815,7 +1710,7 @@ public:
 	boost::statechart::custom_reaction< RemoteRecoveryReserved >,
 	boost::statechart::transition< AllRemotesReserved, Recovering >
 	> reactions;
-      set<pg_shard_t>::const_iterator remote_recovery_reservation_it;
+      set<pg_shard_t>::const_iterator acting_osd_it;
       WaitRemoteRecoveryReserved(my_context ctx);
       boost::statechart::result react(const RemoteRecoveryReserved &evt);
       void exit();
@@ -1949,23 +1844,10 @@ public:
 
     RecoveryMachine machine;
     PG *pg;
-
-    /// context passed in by state machine caller
-    RecoveryCtx *orig_ctx;
-
-    /// populated if we are buffering messages pending a flush
-    boost::optional<BufferedRecoveryMessages> messages_pending_flush;
-
-    /**
-     * populated between start_handle() and end_handle(), points into
-     * the message lists for messages_pending_flush while blocking messages
-     * or into orig_ctx otherwise
-     */
-    boost::optional<RecoveryCtx> rctx;
+    RecoveryCtx *rctx;
 
   public:
-    RecoveryState(PG *pg)
-      : machine(this, pg), pg(pg), orig_ctx(0) {
+    RecoveryState(PG *pg) : machine(this, pg), pg(pg), rctx(0) {
       machine.initiate();
     }
 
@@ -1995,16 +1877,11 @@ public:
   // Prevent copying
   PG(const PG& rhs);
   PG& operator=(const PG& rhs);
-  uint64_t peer_features;
 
  public:
   spg_t      get_pgid() const { return info.pgid; }
   int        get_nrep() const { return acting.size(); }
 
-  void reset_peer_features() { peer_features = (uint64_t)-1; }
-  uint64_t get_min_peer_features() { return peer_features; }
-  void apply_peer_features(uint64_t f) { peer_features &= f; }
-
   void init_primary_up_acting(
     const vector<int> &newup,
     const vector<int> &newacting,
@@ -2074,11 +1951,11 @@ public:
 
   void init(
     int role,
-    vector<int>& up,
+    const vector<int>& up,
     int up_primary,
-    vector<int>& acting,
+    const vector<int>& acting,
     int acting_primary,
-    pg_history_t& history,
+    const pg_history_t& history,
     pg_interval_map_t& pim,
     bool backfill,
     ObjectStore::Transaction *t);
@@ -2108,17 +1985,14 @@ public:
 
   void add_log_entry(pg_log_entry_t& e, bufferlist& log_bl);
   void append_log(
-    vector<pg_log_entry_t>& logv,
-    eversion_t trim_to,
-    eversion_t trim_rollback_to,
-    ObjectStore::Transaction &t,
+    vector<pg_log_entry_t>& logv, eversion_t trim_to, ObjectStore::Transaction &t,
     bool transaction_applied = true);
   bool check_log_for_corruption(ObjectStore *store);
   void trim_peers();
 
   std::string get_corrupt_pg_log_name() const;
   static int read_info(
-    ObjectStore *store, const coll_t coll,
+    ObjectStore *store, const coll_t &coll,
     bufferlist &bl, pg_info_t &info, map<epoch_t,pg_interval_t> &past_intervals,
     hobject_t &biginfo_oid, hobject_t &infos_oid,
     interval_set<snapid_t>  &snap_collections, __u8 &);
@@ -2141,7 +2015,6 @@ public:
   /// share new pg log entries after a pg is active
   void share_pg_log();
 
-  void reset_interval_flush();
   void start_peering_interval(
     const OSDMapRef lastmap,
     const vector<int>& newup, int up_primary,
@@ -2196,6 +2069,12 @@ public:
   void take_waiters();
   void queue_peering_event(CephPeeringEvtRef evt);
   void handle_peering_event(CephPeeringEvtRef evt, RecoveryCtx *rctx);
+  void queue_notify(epoch_t msg_epoch, epoch_t query_epoch,
+		    pg_shard_t from, pg_notify_t& i);
+  void queue_info(epoch_t msg_epoch, epoch_t query_epoch,
+		  pg_shard_t from, pg_info_t& i);
+  void queue_log(epoch_t msg_epoch, epoch_t query_epoch, pg_shard_t from,
+		 MOSDPGLog *msg);
   void queue_query(epoch_t msg_epoch, epoch_t query_epoch,
 		   pg_shard_t from, const pg_query_t& q);
   void queue_null(epoch_t msg_epoch, epoch_t query_epoch);
