@@ -666,6 +666,15 @@ bool RGWAccessKeyPool::check_existing_key(RGWUserAdminOpState& op_state)
 
   switch (key_type) {
   case KEY_TYPE_SWIFT:
+    kiter = swift_keys->find(kid);
+
+    existing_key = (kiter != swift_keys->end());
+    if (existing_key)
+      break;
+
+    if (swift_kid.empty())
+      return false;
+
     kiter = swift_keys->find(swift_kid);
 
     existing_key = (kiter != swift_keys->end());
@@ -728,9 +737,6 @@ int RGWAccessKeyPool::check_op(RGWUserAdminOpState& op_state,
     return -EACCES;
   }
 
-  std::string access_key = op_state.get_access_key();
-  std::string secret_key = op_state.get_secret_key();
-
   int32_t key_type = op_state.get_key_type();
 
   // if a key type wasn't specified set it to s3
@@ -739,8 +745,9 @@ int RGWAccessKeyPool::check_op(RGWUserAdminOpState& op_state,
 
   op_state.set_key_type(key_type);
 
-  /* see if the access key or secret key was specified */
-  if (key_type == KEY_TYPE_S3 && !op_state.will_gen_access() && access_key.empty()) {
+  /* see if the access key was specified */
+  if (key_type == KEY_TYPE_S3 && !op_state.will_gen_access() && 
+      op_state.get_access_key().empty()) {
     set_err_msg(err_msg, "empty access key");
     return -EINVAL;
   }
@@ -766,7 +773,6 @@ int RGWAccessKeyPool::generate_key(RGWUserAdminOpState& op_state, std::string *e
   int key_type = op_state.get_key_type();
   bool gen_access = op_state.will_gen_access();
   bool gen_secret = op_state.will_gen_secret();
-  std::string subuser = op_state.get_subuser();
 
   if (!keys_allowed) {
     set_err_msg(err_msg, "access keys not allowed for this user");
@@ -836,7 +842,7 @@ int RGWAccessKeyPool::generate_key(RGWUserAdminOpState& op_state, std::string *e
     } while (!rgw_get_user_info_by_access_key(store, id, duplicate_check));
   }
 
-  if (key_type == KEY_TYPE_SWIFT) {
+  if (key_type == KEY_TYPE_SWIFT && gen_access) {
     id = op_state.build_default_swift_kid();
     if (id.empty()) {
       set_err_msg(err_msg, "empty swift access key");
@@ -1914,11 +1920,8 @@ int RGWUser::execute_modify(RGWUserAdminOpState& op_state, std::string *err_msg)
 
   std::string old_email = old_info.user_email;
   if (!op_email.empty()) {
-    bool same_email = false;
-    same_email = (old_email.compare(op_email) == 0);
-
     // make sure we are not adding a duplicate email
-    if (!same_email) {
+    if (old_email.compare(op_email) != 0) {
       ret = rgw_get_user_info_by_email(store, op_email, duplicate_check);
       if (ret >= 0 && duplicate_check.user_id != user_id) {
         set_err_msg(err_msg, "cannot add duplicate email");
